@@ -115,7 +115,7 @@ typedef struct {
  * PosOskWidget:
  * @name: The name of the layout, e.g. `de`, `us`, `de+ch`
  * @display_name: The display name of the layout, e.g. `German`, `English (US)`
- * @langauge: The language of the layout e.g. `de`, `en`
+ * @language: The language of the layout e.g. `de`, `en`
  * @region: The region the layout is for e.g. `at` for language `de` or `us` for language `en`.
  *
  * Renders the keyboard and reacts to keypresses by signal emissions.
@@ -132,6 +132,7 @@ struct _PosOskWidget {
   PosOskWidgetMode     mode;
   /* Contains pointers to key symbols (keys have ownership) */
   GPtrArray           *symbols;
+  gboolean             caps_lock;
 
   char                *name;
   char                *display_name;
@@ -436,7 +437,7 @@ parse_row (PosOskWidget      *self,
     } else if (JSON_NODE_HOLDS (key_node, JSON_NODE_OBJECT)) {
       key = POS_OSK_KEY (json_gobject_deserialize (POS_TYPE_OSK_KEY, key_node));
     } else {
-      g_warning ("Unparseable key in row %d pos %d", r, i);
+      g_warning ("Unparsable key in row %d pos %d", r, i);
       continue;
     }
 
@@ -727,8 +728,9 @@ switch_layer (PosOskWidget *self, PosOskKey *key)
         break;
       }
     }
+    self->caps_lock = FALSE;
     /* Reset caps layer on every (non toggle) key press */
-  } else if (new_layer == POS_OSK_WIDGET_LAYER_CAPS) {
+  } else if (new_layer == POS_OSK_WIDGET_LAYER_CAPS && !self->caps_lock) {
     new_layer = POS_OSK_WIDGET_LAYER_NORMAL;
   }
 
@@ -736,6 +738,21 @@ switch_layer (PosOskWidget *self, PosOskKey *key)
 }
 
 
+static void
+set_caps_lock (PosOskWidget *self, gboolean caps_lock)
+{
+  PosOskWidgetLayer layer;
+
+  if (self->caps_lock == caps_lock)
+    return;
+
+  self->caps_lock = caps_lock;
+
+  layer = self->caps_lock ? POS_OSK_WIDGET_LAYER_CAPS : POS_OSK_WIDGET_LAYER_NORMAL;
+  pos_osk_widget_set_layer (self, layer);
+}
+
+
 static PosOskKey *
 pos_osk_widget_locate_key (PosOskWidget *self, double x, double y)
 {
@@ -1072,6 +1089,14 @@ on_long_pressed (GtkGestureLongPress *gesture, double x, double y, gpointer user
     return;
   }
 
+  if (pos_osk_key_get_use (key) == POS_OSK_KEY_USE_TOGGLE &&
+      pos_osk_key_get_layer (key) == POS_OSK_WIDGET_LAYER_CAPS) {
+    g_debug ("Enabling caps lock");
+    set_caps_lock (self, TRUE);
+    pos_osk_widget_cancel_press (self);
+    return;
+  }
+
   symbols = pos_osk_key_get_symbols (key);
   if (symbols == NULL || symbols[0] == NULL)
     return;
@@ -1426,7 +1451,7 @@ pos_osk_widget_class_init (PosOskWidgetClass *klass)
   /**
    * PosOskWidget:features
    *
-   * Feauture flags to configure this widget
+   * Feature flags to configure this widget
    */
   props[PROP_FEATURES] =
     g_param_spec_flags ("features", "", "",
@@ -1719,7 +1744,7 @@ parse_lang (PosOskWidget *self, const char *layout, const char *variant)
  *
  * Sets the widgets keyboard layout.
  *
- * Retruns: %TRUE on success, %FALSE otherwise.
+ * Returns: %TRUE on success, %FALSE otherwise.
  */
 gboolean
 pos_osk_widget_set_layout (PosOskWidget *self,
